clear all, close all

% Constants.
dim = 2;    % Dimensions.
N = 120;      % Number of particles.
m = 0.1;   % Mass per particle [kg].
vd = 1 - 1e-3; % Velocity damping factor.
cd = 0.8;  % Collision damping factor.

plot_particles = true;  % Particles or isolines of the color field?
create_povray = true;   % Create input files for POV-Ray?

fps = 24;      % Frames per second for the movie.
perframe = 10; % Make this many iterations per movie frame.
dt = 0.00025;  % Timestep [s].
t_tot = 1;  % Total time [s].

if (dim==2)
   g = [0; -9.81];   % Gravity [m/s^2].
else
   g = [0; 0; -9.81];
end
geometry = [0 1 1 6 6 7 7 0 0;
            7 7 1 1 7 7 0 0 7];  % 2D geometry data.
geometry = 0.01 * geometry;         
height = geometry(2,1);
bottomheight = geometry(2,3);
leftwall = geometry(1,2);
rightwall = geometry(1,4);

% Initialize.
particles = initialize_particles(dim,N);
velocities = initialize_velocities(dim,N);
t = 0;

plot(geometry(1,:),geometry(2,:),'bla','LineWidth',3)
axis equal, axis([-0.5 7.5 0 10] * 0.01)
hold on
if plot_particles
   h = plot(particles(1,:),particles(2,:),'b.','MarkerSize',10,...
      'EraseMode','xor');
else
   % Set up grid for isolines.
   [X,Y] = meshgrid(linspace(0.01,0.06,25),linspace(0.01,0.1,45));
   % Calculate color field.
   Z = getcolorfield(X,Y,particles,m);
   [C,h] = contour(X,Y,Z,1,'EraseMode','xor','Fill','on');
end
if create_povray
   povray(particles,0)
end

counter = 1;
frame = 1;

while (t < t_tot)
   % Calculate forces: Lennard-Jones plus gravity.
   forces = zeros(size(velocities));
   for i = 1:N
      for j = [1:i-1 i+1:N]
         forces(:,i) = forces(:,i) +...
            lennard_jones(particles(:,i),particles(:,j));
      end
      forces(:,i) = forces(:,i) + g*m;
   end
   
   % Update velocities.
   % Update positions (leap frog: use new velocities).
   for i = 1:N
      velocities(:,i) = vd * (velocities(:,i) + dt*forces(:,i)/m);
      particles(:,i) = particles(:,i) + dt*velocities(:,i);
   end
   
   t = t + dt;
   fprintf('\n%.2f%% done\n',t/t_tot*100)

   % Collision detection: Project onto surface, mirror velocity.
   for i = 1:N
      if (particles(2,i) < height)
         if (particles(1,i) <= leftwall) % Check collision with left wall.
            particles(1,i) = leftwall;  % Set back on wall.
            velocities(1,i) = -cd*velocities(1,i);
         elseif (particles(1,i) >= rightwall) % Check collision with right wall.
            particles(1,i) = rightwall;  % Set back on wall.
            velocities(1,i) = -cd*velocities(1,i);
         end
         if (particles(2,i) < bottomheight) % Check collision with bottom.
            particles(2,i) = bottomheight;  % Set back to bottom level.
            velocities(2,i) = -cd*velocities(2,i); % Mirror velocity.
         end
      end
   end
%    if plot_particles
%       set(h,'XData',particles(1,:),'YData',particles(2,:));
%    end
%    pause(dt)
%    drawnow
   
   if (rem(counter,perframe)==0) % It's time for a frame
      % Update plot.
      if plot_particles
         set(h,'XData',particles(1,:),'YData',particles(2,:));
      else
         Z = getcolorfield(X,Y,particles,m);
         set(h,'ZData',Z)
      end
      if create_povray
         povray(particles,frame)
      end
      drawnow
      fluidmovie(frame) = getframe;
      frame = frame + 1;
   end
   counter = counter + 1;
end

mov = avifile('movie.avi','fps',24,'quality',90);
for i = 1:length(fluidmovie)
   mov = addframe(mov,fluidmovie(i));
end
mov = close(mov);

% movie(fluidmovie,1,fps);